@@ -210,14 +210,15 @@ class DecisionTree :
    * @param weights Weights of all the labels
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<typename MatType, typename LabelsType>
-  void Train(MatType data,
-             const data::DatasetInfo& datasetInfo,
-             LabelsType labels,
-             const size_t numClasses,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7);
+  double Train(MatType data,
+               const data::DatasetInfo& datasetInfo,
+               LabelsType labels,
+               const size_t numClasses,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7);
 
   /**
    * Train the decision tree on the given data, assuming that all dimensions are
@@ -233,13 +234,14 @@ class DecisionTree :
    * @param weights Weights of all the labels
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<typename MatType, typename LabelsType>
-  void Train(MatType data,
-             LabelsType labels,
-             const size_t numClasses,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7);
+  double Train(MatType data,
+               LabelsType labels,
+               const size_t numClasses,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7);
 
   /**
    * Train the decision tree on the given weighted data.  This will overwrite
@@ -258,17 +260,18 @@ class DecisionTree :
    * @param weights Weights of all the labels
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<typename MatType, typename LabelsType, typename WeightsType>
-  void Train(MatType data,
-             const data::DatasetInfo& datasetInfo,
-             LabelsType labels,
-             const size_t numClasses,
-             WeightsType weights,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7,
-             const std::enable_if_t<arma::is_arma_type<typename
-                 std::remove_reference<WeightsType>::type>::value>* = 0);
+  double Train(MatType data,
+               const data::DatasetInfo& datasetInfo,
+               LabelsType labels,
+               const size_t numClasses,
+               WeightsType weights,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7,
+               const std::enable_if_t<arma::is_arma_type<typename
+                   std::remove_reference<WeightsType>::type>::value>* = 0);
 
   /**
    * Train the decision tree on the given weighted data, assuming that all
@@ -285,16 +288,17 @@ class DecisionTree :
    * @param weights Weights of all the labels
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<typename MatType, typename LabelsType, typename WeightsType>
-  void Train(MatType data,
-             LabelsType labels,
-             const size_t numClasses,
-             WeightsType weights,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7,
-             const std::enable_if_t<arma::is_arma_type<typename
-                 std::remove_reference<WeightsType>::type>::value>* = 0);
+  double Train(MatType data,
+               LabelsType labels,
+               const size_t numClasses,
+               WeightsType weights,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7,
+               const std::enable_if_t<arma::is_arma_type<typename
+                   std::remove_reference<WeightsType>::type>::value>* = 0);
 
   /**
    * Classify the given point, using the entire tree.  The predicted label is
@@ -421,17 +425,18 @@ class DecisionTree :
    * @param numClasses Number of classes in the dataset.
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<bool UseWeights, typename MatType>
-  void Train(MatType& data,
-             const size_t begin,
-             const size_t count,
-             const data::DatasetInfo& datasetInfo,
-             arma::Row<size_t>& labels,
-             const size_t numClasses,
-             arma::rowvec& weights,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7);
+  double Train(MatType& data,
+               const size_t begin,
+               const size_t count,
+               const data::DatasetInfo& datasetInfo,
+               arma::Row<size_t>& labels,
+               const size_t numClasses,
+               arma::rowvec& weights,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7);
 
   /**
    * Corresponding to the public Train() method, this method is designed for
@@ -446,16 +451,17 @@ class DecisionTree :
    * @param numClasses Number of classes in the dataset.
    * @param minimumLeafSize Minimum number of points in each leaf node.
    * @param minimumGainSplit Minimum gain for the node to split.
+   * @return The final entropy of decision tree.
    */
   template<bool UseWeights, typename MatType>
-  void Train(MatType& data,
-             const size_t begin,
-             const size_t count,
-             arma::Row<size_t>& labels,
-             const size_t numClasses,
-             arma::rowvec& weights,
-             const size_t minimumLeafSize = 10,
-             const double minimumGainSplit = 1e-7);
+  double Train(MatType& data,
+               const size_t begin,
+               const size_t count,
+               arma::Row<size_t>& labels,
+               const size_t numClasses,
+               arma::rowvec& weights,
+               const size_t minimumLeafSize = 10,
+               const double minimumGainSplit = 1e-7);
 };
 
 /**
