@@ -14,10 +14,24 @@
 #include <mlpack/core.hpp>
 #include <mlpack/methods/cf/cf.hpp>
 #include <mlpack/methods/cf/decomposition_policies/batch_svd_method.hpp>
+#include <mlpack/methods/cf/decomposition_policies/bias_svd_method.hpp>
 #include <mlpack/methods/cf/decomposition_policies/randomized_svd_method.hpp>
 #include <mlpack/methods/cf/decomposition_policies/regularized_svd_method.hpp>
 #include <mlpack/methods/cf/decomposition_policies/svd_complete_method.hpp>
 #include <mlpack/methods/cf/decomposition_policies/svd_incomplete_method.hpp>
+#include <mlpack/methods/cf/decomposition_policies/svdplusplus_method.hpp>
+#include <mlpack/methods/cf/normalization/no_normalization.hpp>
+#include <mlpack/methods/cf/normalization/overall_mean_normalization.hpp>
+#include <mlpack/methods/cf/normalization/user_mean_normalization.hpp>
+#include <mlpack/methods/cf/normalization/item_mean_normalization.hpp>
+#include <mlpack/methods/cf/normalization/z_score_normalization.hpp>
+#include <mlpack/methods/cf/normalization/combined_normalization.hpp>
+#include <mlpack/methods/cf/neighbor_search_policies/lmetric_search.hpp>
+#include <mlpack/methods/cf/neighbor_search_policies/cosine_search.hpp>
+#include <mlpack/methods/cf/neighbor_search_policies/pearson_search.hpp>
+#include <mlpack/methods/cf/interpolation_policies/average_interpolation.hpp>
+#include <mlpack/methods/cf/interpolation_policies/similarity_interpolation.hpp>
+#include <mlpack/methods/cf/interpolation_policies/regression_interpolation.hpp>
 
 #include <iostream>
 
@@ -31,107 +45,13 @@ using namespace mlpack;
 using namespace mlpack::cf;
 using namespace std;
 
-/**
- * Make sure that correct number of recommendations are generated when query
- * set. Default case.
- */
-template<typename DecompositionPolicy>
-void GetRecommendationsAllUsers(bool cleanData = true)
+// Get train and test datasets.
+static void GetDatasets(arma::mat& dataset, arma::mat& savedCols)
 {
-  DecompositionPolicy decomposition;
-  // Dummy number of recommendations.
-  size_t numRecs = 3;
-  // GroupLensSmall.csv dataset has 200 users.
-  size_t numUsers = 200;
-
-  // Matrix to save recommendations into.
-  arma::Mat<size_t> recommendations;
-
-  // Load GroupLens data.
-  arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
-
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
-
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
-  CFType c(dataset, decomposition, 5, 5, 70);
-
-  // Generate recommendations when query set is not specified.
-  c.GetRecommendations(numRecs, recommendations);
-
-  // Check if correct number of recommendations are generated.
-  BOOST_REQUIRE_EQUAL(recommendations.n_rows, numRecs);
-
-  // Check if recommendations are generated for all users.
-  BOOST_REQUIRE_EQUAL(recommendations.n_cols, numUsers);
-}
-
-/**
- * Make sure that the recommendations are generated for queried users only.
- */
-template<typename DecompositionPolicy>
-void GetRecommendationsQueriedUser(bool cleanData = true)
-{
-  DecompositionPolicy decomposition;
-  // Number of users that we will search for recommendations for.
-  size_t numUsers = 10;
-
-  // Default number of recommendations.
-  size_t numRecsDefault = 5;
-
-  // Create dummy query set.
-  arma::Col<size_t> users = arma::zeros<arma::Col<size_t> >(numUsers, 1);
-  for (size_t i = 0; i < numUsers; i++)
-    users(i) = i;
-
-  // Matrix to save recommendations into.
-  arma::Mat<size_t> recommendations;
-
-  // Load GroupLens data.
-  arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
-
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
-
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
-
-  CFType c(dataset, decomposition, 5, 5, 70);
-
-  // Generate recommendations when query set is specified.
-  c.GetRecommendations(numRecsDefault, recommendations, users);
-
-  // Check if correct number of recommendations are generated.
-  BOOST_REQUIRE_EQUAL(recommendations.n_rows, numRecsDefault);
-
-  // Check if recommendations are generated for the right number of users.
-  BOOST_REQUIRE_EQUAL(recommendations.n_cols, numUsers);
-}
-
-/**
- * Make sure recommendations that are generated are reasonably accurate.
- */
-template<typename DecompositionPolicy>
-void RecommendationAccuracy(bool cleanData = true)
-{
-  DecompositionPolicy decomposition;
-  // Load the GroupLens dataset; then, we will remove some values from it.
-  arma::mat dataset;
   data::Load("GroupLensSmall.csv", dataset);
+  savedCols.set_size(3, 50);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
   savedCols.fill(/* random very large value */ 10000000);
   size_t currentCol = 0;
   for (size_t i = 0; i < dataset.n_cols; ++i)
@@ -153,8 +73,8 @@ void RecommendationAccuracy(bool cleanData = true)
         }
       }
 
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
+      // If this user doesn't already exist in savedCols, add them.
+      // Otherwise ignore this point.
       if (!found)
       {
         savedCols.col(currentCol) = dataset.col(i);
@@ -163,18 +83,96 @@ void RecommendationAccuracy(bool cleanData = true)
       }
     }
   }
+}
 
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
+/**
+ * Make sure that correct number of recommendations are generated when query
+ * set. Default case.
+ */
+template<typename DecompositionPolicy>
+void GetRecommendationsAllUsers()
+{
+  DecompositionPolicy decomposition;
+  // Dummy number of recommendations.
+  size_t numRecs = 3;
+  // GroupLensSmall.csv dataset has 200 users.
+  size_t numUsers = 200;
 
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
+  // Matrix to save recommendations into.
+  arma::Mat<size_t> recommendations;
 
-  CFType c(dataset, decomposition, 5, 5, 70);
+  // Load GroupLens data.
+  arma::mat dataset;
+  data::Load("GroupLensSmall.csv", dataset);
+
+  CFType<DecompositionPolicy> c(dataset, decomposition, 5, 5, 30);
+
+  // Generate recommendations when query set is not specified.
+  c.GetRecommendations(numRecs, recommendations);
+
+  // Check if correct number of recommendations are generated.
+  BOOST_REQUIRE_EQUAL(recommendations.n_rows, numRecs);
+
+  // Check if recommendations are generated for all users.
+  BOOST_REQUIRE_EQUAL(recommendations.n_cols, numUsers);
+}
+
+/**
+ * Make sure that the recommendations are generated for queried users only.
+ */
+template<typename DecompositionPolicy>
+void GetRecommendationsQueriedUser()
+{
+  DecompositionPolicy decomposition;
+  // Number of users that we will search for recommendations for.
+  size_t numUsers = 10;
+
+  // Default number of recommendations.
+  size_t numRecsDefault = 5;
+
+  // Create dummy query set.
+  arma::Col<size_t> users = arma::zeros<arma::Col<size_t> >(numUsers, 1);
+  for (size_t i = 0; i < numUsers; i++)
+    users(i) = i;
+
+  // Matrix to save recommendations into.
+  arma::Mat<size_t> recommendations;
+
+  // Load GroupLens data.
+  arma::mat dataset;
+  data::Load("GroupLensSmall.csv", dataset);
+
+  CFType<DecompositionPolicy> c(dataset, decomposition, 5, 5, 30);
+
+  // Generate recommendations when query set is specified.
+  c.GetRecommendations(numRecsDefault, recommendations, users);
+
+  // Check if correct number of recommendations are generated.
+  BOOST_REQUIRE_EQUAL(recommendations.n_rows, numRecsDefault);
+
+  // Check if recommendations are generated for the right number of users.
+  BOOST_REQUIRE_EQUAL(recommendations.n_cols, numUsers);
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate.
+ */
+template<typename DecompositionPolicy,
+         typename NormalizationType = NoNormalization>
+void RecommendationAccuracy()
+{
+  DecompositionPolicy decomposition;
+
+  // Small GroupLens dataset.
+  arma::mat dataset;
+
+  // Save the columns we've removed.
+  arma::mat savedCols;
+
+  GetDatasets(dataset, savedCols);
+
+  CFType<DecompositionPolicy,
+      NormalizationType> c(dataset, decomposition, 5, 5, 30);
 
   // Obtain 150 recommendations for the users in savedCols, and make sure the
   // missing item shows up in most of them.  First, create the list of users,
@@ -215,141 +213,65 @@ void RecommendationAccuracy(bool cleanData = true)
       ++failures;
   }
 
-  // Make sure the right item showed up in at least 1/3 of the recommendations.
-  // GroupLens dataset would give somewhere around a 10% success rate (failures
-  // would be closer to 270).  The failure rate is allowed to be so high because
-  // the dataset used here is pretty small and it is hard to generalize.
+  // Make sure the right item showed up in at least 2/3 of the recommendations.
   BOOST_REQUIRE_LT(failures, 17);
 }
 
 // Make sure that Predict() is returning reasonable results.
-template<typename DecompositionPolicy>
-void CFPredict(bool cleanData = true)
+template<typename DecompositionPolicy,
+         typename NormalizationType = OverallMeanNormalization,
+         typename NeighborSearchPolicy = EuclideanSearch,
+         typename InterpolationPolicy = AverageInterpolation>
+void CFPredict(const double rmseBound = 1.5)
 {
   DecompositionPolicy decomposition;
-  // Load the GroupLens dataset; then, we will remove some values from it.
+
+  // Small GroupLens dataset.
   arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
-  savedCols.fill(/* random very large value */ 10000000);
-  size_t currentCol = 0;
-  for (size_t i = 0; i < dataset.n_cols; ++i)
-  {
-    if (currentCol == 50)
-      break;
+  arma::mat savedCols;
 
-    if (dataset(2, i) > 4.5) // 5-star rating.
-    {
-      // Make sure we don't have this user yet.  This is a slow way to do this
-      // but I don't particularly care here because it's in the tests.
-      bool found = false;
-      for (size_t j = 0; j < currentCol; ++j)
-      {
-        if (savedCols(0, j) == dataset(0, i))
-        {
-          found = true;
-          break;
-        }
-      }
+  GetDatasets(dataset, savedCols);
 
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
-      if (!found)
-      {
-        savedCols.col(currentCol) = dataset.col(i);
-        dataset.shed_col(i);
-        ++currentCol;
-      }
-    }
-  }
-
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
-
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
-
-  CFType c(dataset, decomposition, 5, 5, 70);
+  CFType<DecompositionPolicy,
+      NormalizationType> c(dataset, decomposition, 5, 5, 30);
 
   // Now, for each removed rating, make sure the prediction is... reasonably
   // accurate.
   double totalError = 0.0;
   for (size_t i = 0; i < savedCols.n_cols; ++i)
   {
-    const double prediction = c.Predict(savedCols(0, i), savedCols(1, i));
+    const double prediction = c.template Predict<NeighborSearchPolicy,
+        InterpolationPolicy>(savedCols(0, i), savedCols(1, i));
 
     const double error = std::pow(prediction - savedCols(2, i), 2.0);
     totalError += error;
   }
 
-  totalError = std::sqrt(totalError) / savedCols.n_cols;
+  const double rmse = std::sqrt(totalError / savedCols.n_cols);
 
-  // The mean squared error should be less than one.
-  BOOST_REQUIRE_LT(totalError, 0.6);
+  // The root mean square error should be less than ?.
+  BOOST_REQUIRE_LT(rmse, rmseBound);
 }
 
 // Do the same thing as the previous test, but ensure that the ratings we
 // predict with the batch Predict() are the same as the individual Predict()
 // calls.
 template<typename DecompositionPolicy>
-void BatchPredict(bool cleanData = true)
+void BatchPredict()
 {
   DecompositionPolicy decomposition;
-  // Load the GroupLens dataset; then, we will remove some values from it.
+
+  // Small GroupLens dataset.
   arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
-  savedCols.fill(/* random very large value */ 10000000);
-  size_t currentCol = 0;
-  for (size_t i = 0; i < dataset.n_cols; ++i)
-  {
-    if (currentCol == 50)
-      break;
+  arma::mat savedCols;
 
-    if (dataset(2, i) > 4.5) // 5-star rating.
-    {
-      // Make sure we don't have this user yet.  This is a slow way to do this
-      // but I don't particularly care here because it's in the tests.
-      bool found = false;
-      for (size_t j = 0; j < currentCol; ++j)
-      {
-        if (savedCols(0, j) == dataset(0, i))
-        {
-          found = true;
-          break;
-        }
-      }
+  GetDatasets(dataset, savedCols);
 
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
-      if (!found)
-      {
-        savedCols.col(currentCol) = dataset.col(i);
-        dataset.shed_col(i);
-        ++currentCol;
-      }
-    }
-  }
-
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
-
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
-
-  CFType c(dataset, decomposition, 5, 5, 70);
+  CFType<DecompositionPolicy> c(dataset, decomposition, 5, 5, 30);
 
   // Get predictions for all user/item pairs we held back.
   arma::Mat<size_t> combinations(2, savedCols.n_cols);
@@ -378,52 +300,22 @@ void Train(DecompositionPolicy& decomposition)
   // Generate random data.
   arma::sp_mat randomData;
   randomData.sprandu(100, 100, 0.3);
-  CFType c(randomData, decomposition, 5, 5, 70);
+  CFType<DecompositionPolicy> c(randomData, decomposition, 5, 5, 30);
 
-  // Now retrain with data we know about.
+  // Small GroupLens dataset.
   arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
-  savedCols.fill(/* random very large value */ 10000000);
-  size_t currentCol = 0;
-  for (size_t i = 0; i < dataset.n_cols; ++i)
-  {
-    if (currentCol == 50)
-      break;
+  arma::mat savedCols;
 
-    if (dataset(2, i) > 4.5) // 5-star rating.
-    {
-      // Make sure we don't have this user yet.  This is a slow way to do this
-      // but I don't particularly care here because it's in the tests.
-      bool found = false;
-      for (size_t j = 0; j < currentCol; ++j)
-      {
-        if (savedCols(0, j) == dataset(0, i))
-        {
-          found = true;
-          break;
-        }
-      }
-
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
-      if (!found)
-      {
-        savedCols.col(currentCol) = dataset.col(i);
-        dataset.shed_col(i);
-        ++currentCol;
-      }
-    }
-  }
+  GetDatasets(dataset, savedCols);
 
   // Make data into sparse matrix.
   arma::sp_mat cleanedData;
-  CFType::CleanData(dataset, cleanedData);
+  CFType<DecompositionPolicy>::CleanData(dataset, cleanedData);
 
   // Now retrain.
-  c.Train(dataset, decomposition, 70);
+  c.Train(dataset, decomposition, 30);
 
   // Get predictions for all user/item pairs we held back.
   arma::Mat<size_t> combinations(2, savedCols.n_cols);
@@ -447,53 +339,26 @@ void Train(DecompositionPolicy& decomposition)
  * Make sure we can train an already-trained model and it works okay
  * for policies that use coordinate lists.
  */
-template<>
-void Train<>(RegSVDPolicy& decomposition)
+template<typename DecompositionPolicy>
+void TrainWithCoordinateList(DecompositionPolicy& decomposition)
 {
-  arma::mat randomData = arma::zeros(100, 100);
-  randomData.diag().ones();
-  CFType c(randomData, decomposition, 5, 5, 70);
+  arma::mat randomData(3, 100);
+  randomData.row(0) = arma::linspace<arma::rowvec>(0, 99, 100);
+  randomData.row(1) = arma::linspace<arma::rowvec>(0, 99, 100);
+  randomData.row(2).fill(3);
+  CFType<DecompositionPolicy> c(randomData, decomposition, 5, 5, 30);
 
   // Now retrain with data we know about.
+  // Small GroupLens dataset.
   arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 50 5-star ratings.
-  savedCols.fill(/* random very large value */ 10000000);
-  size_t currentCol = 0;
-  for (size_t i = 0; i < dataset.n_cols; ++i)
-  {
-    if (currentCol == 50)
-      break;
+  arma::mat savedCols;
 
-    if (dataset(2, i) > 4.5) // 5-star rating.
-    {
-      // Make sure we don't have this user yet.  This is a slow way to do this
-      // but I don't particularly care here because it's in the tests.
-      bool found = false;
-      for (size_t j = 0; j < currentCol; ++j)
-      {
-        if (savedCols(0, j) == dataset(0, i))
-        {
-          found = true;
-          break;
-        }
-      }
-
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
-      if (!found)
-      {
-        savedCols.col(currentCol) = dataset.col(i);
-        dataset.shed_col(i);
-        ++currentCol;
-      }
-    }
-  }
+  GetDatasets(dataset, savedCols);
 
   // Now retrain.
-  c.Train(dataset, decomposition, 70);
+  c.Train(dataset, decomposition, 30);
 
   // Get predictions for all user/item pairs we held back.
   arma::Mat<size_t> combinations(2, savedCols.n_cols);
@@ -517,61 +382,22 @@ void Train<>(RegSVDPolicy& decomposition)
  * Make sure we can train a model after using the empty constructor.
  */
 template<typename DecompositionPolicy>
-void EmptyConstructorTrain(bool cleanData = true)
+void EmptyConstructorTrain()
 {
   DecompositionPolicy decomposition;
   // Use default constructor.
-  CFType c;
+  CFType<DecompositionPolicy> c;
 
   // Now retrain with data we know about.
+  // Small GroupLens dataset.
   arma::mat dataset;
-  data::Load("GroupLensSmall.csv", dataset);
 
   // Save the columns we've removed.
-  arma::mat savedCols(3, 50); // Remove 300 5-star ratings.
-  savedCols.fill(/* random very large value */ 10000000);
-  size_t currentCol = 0;
-  for (size_t i = 0; i < dataset.n_cols; ++i)
-  {
-    if (currentCol == 50)
-      break;
+  arma::mat savedCols;
 
-    if (dataset(2, i) > 4.5) // 5-star rating.
-    {
-      // Make sure we don't have this user yet.  This is a slow way to do this
-      // but I don't particularly care here because it's in the tests.
-      bool found = false;
-      for (size_t j = 0; j < currentCol; ++j)
-      {
-        if (savedCols(0, j) == dataset(0, i))
-        {
-          found = true;
-          break;
-        }
-      }
+  GetDatasets(dataset, savedCols);
 
-      // If this user doesn't already exist in savedCols, add them.  Otherwise
-      // ignore this point.
-      if (!found)
-      {
-        savedCols.col(currentCol) = dataset.col(i);
-        dataset.shed_col(i);
-        ++currentCol;
-      }
-    }
-  }
-
-  if (cleanData)
-  {
-    // Make data into sparse matrix.
-    arma::sp_mat cleanedData;
-    CFType::CleanData(dataset, cleanedData);
-
-    arma::sp_mat dataset;
-    dataset = cleanedData;
-  }
-
-  c.Train(dataset, decomposition, 70);
+  c.Train(dataset, decomposition, 30);
 
   // Get predictions for all user/item pairs we held back.
   arma::Mat<size_t> combinations(2, savedCols.n_cols);
@@ -595,7 +421,8 @@ void EmptyConstructorTrain(bool cleanData = true)
 /**
  * Ensure we can load and save the CF model.
  */
-template<typename DecompositionPolicy>
+template<typename DecompositionPolicy,
+         typename NormalizationType = NoNormalization>
 void Serialization()
 {
   DecompositionPolicy decomposition;
@@ -604,16 +431,21 @@ void Serialization()
   data::Load("GroupLensSmall.csv", dataset);
 
   arma::sp_mat cleanedData;
-  CFType::CleanData(dataset, cleanedData);
+  CFType<DecompositionPolicy,
+      NormalizationType>::CleanData(dataset, cleanedData);
 
-  CFType c(cleanedData, decomposition, 5, 5, 70);
+  CFType<DecompositionPolicy,
+      NormalizationType> c(cleanedData, decomposition, 5, 5, 30);
 
   arma::sp_mat randomData;
   randomData.sprandu(100, 100, 0.3);
 
-  CFType cXml(randomData, decomposition, 5, 5, 70);
-  CFType cBinary;
-  CFType cText(cleanedData, decomposition, 5, 5, 70);
+  CFType<DecompositionPolicy,
+      NormalizationType> cXml(randomData, decomposition, 5, 5, 30);
+  CFType<DecompositionPolicy,
+      NormalizationType> cBinary;
+  CFType<DecompositionPolicy,
+      NormalizationType> cText(cleanedData, decomposition, 5, 5, 30);
 
   SerializeObjectAll(c, cXml, cText, cBinary);
 
@@ -627,8 +459,10 @@ void Serialization()
   BOOST_REQUIRE_EQUAL(c.Rank(), cBinary.Rank());
   BOOST_REQUIRE_EQUAL(c.Rank(), cText.Rank());
 
-  CheckMatrices(c.W(), cXml.W(), cBinary.W(), cText.W());
-  CheckMatrices(c.H(), cXml.H(), cBinary.H(), cText.H());
+  CheckMatrices(c.Decomposition().W(), cXml.Decomposition().W(),
+      cBinary.Decomposition().W(), cText.Decomposition().W());
+  CheckMatrices(c.Decomposition().H(), cXml.Decomposition().H(),
+      cBinary.Decomposition().H(), cText.Decomposition().H());
 
   BOOST_REQUIRE_EQUAL(c.CleanedData().n_rows, cXml.CleanedData().n_rows);
   BOOST_REQUIRE_EQUAL(c.CleanedData().n_rows, cBinary.CleanedData().n_rows);
@@ -690,7 +524,7 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersRandSVDTest)
  */
 BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersRegSVDTest)
 {
-  GetRecommendationsAllUsers<RegSVDPolicy>(false);
+  GetRecommendationsAllUsers<RegSVDPolicy>();
 }
 
 /**
@@ -730,6 +564,24 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersSVDIncompleteTest)
   GetRecommendationsAllUsers<SVDIncompletePolicy>();
 }
 
+/**
+ * Make sure that correct number of recommendations are generated when query
+ * set for Bias SVD method.
+ */
+BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersBiasSVDTest)
+{
+  GetRecommendationsAllUsers<BiasSVDPolicy>();
+}
+
+/**
+ * Make sure that correct number of recommendations are generated when query
+ * set for SVDPlusPlus method.
+ */
+BOOST_AUTO_TEST_CASE(CFGetRecommendationsAllUsersSVDPPTest)
+{
+  GetRecommendationsAllUsers<SVDPlusPlusPolicy>();
+}
+
 /**
  * Make sure that the recommendations are generated for queried users only
  * for randomized SVD.
@@ -745,7 +597,7 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserRandSVDTest)
  */
 BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserRegSVDTest)
 {
-  GetRecommendationsQueriedUser<RegSVDPolicy>(false);
+  GetRecommendationsQueriedUser<RegSVDPolicy>();
 }
 
 /**
@@ -784,6 +636,24 @@ BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserSVDIncompleteTest)
   GetRecommendationsQueriedUser<SVDIncompletePolicy>();
 }
 
+/**
+ * Make sure that the recommendations are generated for queried users only
+ * for Bias SVD method.
+ */
+BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserBiasSVDTest)
+{
+  GetRecommendationsQueriedUser<BiasSVDPolicy>();
+}
+
+/**
+ * Make sure that the recommendations are generated for queried users only
+ * for SVDPlusPlus method.
+ */
+BOOST_AUTO_TEST_CASE(CFGetRecommendationsQueriedUserSVDPPTest)
+{
+  GetRecommendationsQueriedUser<SVDPlusPlusPolicy>();
+}
+
 /**
  * Make sure recommendations that are generated are reasonably accurate
  * for randomized SVD.
@@ -799,7 +669,7 @@ BOOST_AUTO_TEST_CASE(RecommendationAccuracyRandSVDTest)
  */
 BOOST_AUTO_TEST_CASE(RecommendationAccuracyRegSVDTest)
 {
-  RecommendationAccuracy<RegSVDPolicy>(false);
+  RecommendationAccuracy<RegSVDPolicy>();
 }
 
 /**
@@ -838,6 +708,26 @@ BOOST_AUTO_TEST_CASE(RecommendationAccuracySVDIncompleteTest)
   RecommendationAccuracy<SVDIncompletePolicy>();
 }
 
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for Bias SVD method.
+ */ 
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyBiasSVDTest)
+{
+  RecommendationAccuracy<BiasSVDPolicy>();
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for SVDPlusPlus method.
+ */
+// This test is commented out because it fails and we haven't solved it yet.
+// Please refer to issue #1501 for more info about this test.
+// BOOST_AUTO_TEST_CASE(RecommendationAccuracySVDPPTest)
+// {
+//   RecommendationAccuracy<SVDPlusPlusPolicy>();
+// }
+
 // Make sure that Predict() is returning reasonable results for randomized SVD.
 BOOST_AUTO_TEST_CASE(CFPredictRandSVDTest)
 {
@@ -847,7 +737,7 @@ BOOST_AUTO_TEST_CASE(CFPredictRandSVDTest)
 // Make sure that Predict() is returning reasonable results for regularized SVD.
 BOOST_AUTO_TEST_CASE(CFPredictRegSVDTest)
 {
-  CFPredict<RegSVDPolicy>(false);
+  CFPredict<RegSVDPolicy>();
 }
 
 // Make sure that Predict() is returning reasonable results for batch SVD.
@@ -880,6 +770,24 @@ BOOST_AUTO_TEST_CASE(CFPredictSVDIncompleteTest)
   CFPredict<SVDIncompletePolicy>();
 }
 
+/**
+ * Make sure that Predict() is returning reasonable results for Bias SVD
+ * method.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictBiasSVDTest)
+{
+  CFPredict<BiasSVDPolicy>();
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for SVDPlusPlus
+ * method.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictSVDPPTest)
+{
+  CFPredict<SVDPlusPlusPolicy>();
+}
+
 // Compare batch Predict() and individual Predict() for randomized SVD.
 BOOST_AUTO_TEST_CASE(CFBatchPredictRandSVDTest)
 {
@@ -889,7 +797,7 @@ BOOST_AUTO_TEST_CASE(CFBatchPredictRandSVDTest)
 // Compare batch Predict() and individual Predict() for regularized SVD.
 BOOST_AUTO_TEST_CASE(CFBatchPredictRegSVDTest)
 {
-  BatchPredict<RegSVDPolicy>(false);
+  BatchPredict<RegSVDPolicy>();
 }
 
 // Compare batch Predict() and individual Predict() for batch SVD.
@@ -918,6 +826,20 @@ BOOST_AUTO_TEST_CASE(CFBatchPredictSVDIncompleteTest)
   BatchPredict<SVDIncompletePolicy>();
 }
 
+// Compare batch Predict() and individual Predict() for
+// Bias SVD method.
+BOOST_AUTO_TEST_CASE(CFBatchPredictBiasSVDTest)
+{
+  BatchPredict<BiasSVDPolicy>();
+}
+
+// Compare batch Predict() and individual Predict() for
+// SVDPlusPlus method.
+BOOST_AUTO_TEST_CASE(CFBatchPredictSVDPPTest)
+{
+  BatchPredict<SVDPlusPlusPolicy>();
+}
+
 /**
  * Make sure we can train an already-trained model and it works okay for
  * randomized SVD.
@@ -935,7 +857,7 @@ BOOST_AUTO_TEST_CASE(TrainRandSVDTest)
 BOOST_AUTO_TEST_CASE(TrainRegSVDTest)
 {
   RegSVDPolicy decomposition;
-  Train(decomposition);
+  TrainWithCoordinateList(decomposition);
 }
 
 /**
@@ -978,6 +900,26 @@ BOOST_AUTO_TEST_CASE(TrainSVDIncompleteTest)
   Train(decomposition);
 }
 
+/**
+ * Make sure we can train an already-trained model and it works okay for
+ * BiasSVD method.
+ */
+BOOST_AUTO_TEST_CASE(TrainBiasSVDTest)
+{
+  BiasSVDPolicy decomposition;
+  TrainWithCoordinateList(decomposition);
+}
+
+/**
+ * Make sure we can train an already-trained model and it works okay for
+ * SVDPlusPlus method.
+ */
+BOOST_AUTO_TEST_CASE(TrainSVDPPTest)
+{
+  SVDPlusPlusPolicy decomposition;
+  TrainWithCoordinateList(decomposition);
+}
+
 /**
  * Make sure we can train a model after using the empty constructor when
  * using randomized SVD.
@@ -993,7 +935,7 @@ BOOST_AUTO_TEST_CASE(EmptyConstructorTrainRandSVDTest)
  */
 BOOST_AUTO_TEST_CASE(EmptyConstructorTrainRegSVDTest)
 {
-  EmptyConstructorTrain<RegSVDPolicy>(false);
+  EmptyConstructorTrain<RegSVDPolicy>();
 }
 
 /**
@@ -1072,4 +1014,218 @@ BOOST_AUTO_TEST_CASE(SerializationSVDIncompleteTest)
   Serialization<SVDIncompletePolicy>();
 }
 
+/**
+ * Make sure that Predict() is returning reasonable results for NMF and
+ * OverallMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictOverallMeanNormalization)
+{
+  CFPredict<NMFPolicy, OverallMeanNormalization>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for NMF and
+ * UserMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictUserMeanNormalization)
+{
+  CFPredict<NMFPolicy, UserMeanNormalization>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for NMF and
+ * ItemMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictItemMeanNormalization)
+{
+  CFPredict<NMFPolicy, ItemMeanNormalization>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for NMF and
+ * ZScoreNormalization.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictZScoreNormalization)
+{
+  CFPredict<NMFPolicy, ZScoreNormalization>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for NMF and
+ * CombinedNormalization<OverallMeanNormalization, UserMeanNormalization,
+ * ItemMeanNormalization>.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictCombinedNormalization)
+{
+  CFPredict<NMFPolicy,
+            CombinedNormalization<
+                OverallMeanNormalization,
+                UserMeanNormalization,
+                ItemMeanNormalization>>(2.0);
+}
+
+/**
+ * Make sure that Predict() works with NoNormalization.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictNoNormalization)
+{
+  CFPredict<RegSVDPolicy, NoNormalization>(2.0);
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for OverallMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyOverallMeanNormalizationTest)
+{
+  RecommendationAccuracy<NMFPolicy, OverallMeanNormalization>();
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for UserMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyUserMeanNormalizationTest)
+{
+  RecommendationAccuracy<NMFPolicy, UserMeanNormalization>();
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for ItemMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyItemMeanNormalizationTest)
+{
+  RecommendationAccuracy<NMFPolicy, ItemMeanNormalization>();
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for ZScoreNormalization.
+ */
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyZScoreNormalizationTest)
+{
+  RecommendationAccuracy<NMFPolicy, ZScoreNormalization>();
+}
+
+/**
+ * Make sure recommendations that are generated are reasonably accurate
+ * for CombinedNormalization.
+ */
+BOOST_AUTO_TEST_CASE(RecommendationAccuracyCombinedNormalizationTest)
+{
+  RecommendationAccuracy<NMFPolicy,
+                         CombinedNormalization<
+                           OverallMeanNormalization,
+                           UserMeanNormalization,
+                           ItemMeanNormalization>>();
+}
+
+/**
+ * Ensure we can load and save the CF model using OverallMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(SerializationOverallMeanNormalizationTest)
+{
+  Serialization<NMFPolicy, OverallMeanNormalization>();
+}
+
+/**
+ * Ensure we can load and save the CF model using UserMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(SerializationUserMeanNormalizationTest)
+{
+  Serialization<NMFPolicy, UserMeanNormalization>();
+}
+
+/**
+ * Ensure we can load and save the CF model using ItemMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(SerializationItemMeanNormalizationTest)
+{
+  Serialization<NMFPolicy, ItemMeanNormalization>();
+}
+
+/**
+ * Ensure we can load and save the CF model using ZScoreMeanNormalization.
+ */
+BOOST_AUTO_TEST_CASE(SerializationZScoreNormalizationTest)
+{
+  Serialization<NMFPolicy, ZScoreNormalization>();
+}
+
+/**
+ * Ensure we can load and save the CF model using CombinedNormalization.
+ */
+BOOST_AUTO_TEST_CASE(SerializationCombinedNormalizationTest)
+{
+  Serialization<NMFPolicy,
+                CombinedNormalization<
+                    OverallMeanNormalization,
+                    UserMeanNormalization,
+                    ItemMeanNormalization>>();
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * EuclideanSearch.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictEuclideanSearch)
+{
+  CFPredict<NMFPolicy, OverallMeanNormalization, EuclideanSearch>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * CosineSearch.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictCosineSearch)
+{
+  CFPredict<NMFPolicy, OverallMeanNormalization, CosineSearch>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * PearsonSearch.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictPearsonSearch)
+{
+  CFPredict<NMFPolicy, OverallMeanNormalization, PearsonSearch>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * AverageInterpolation.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictAverageInterpolation)
+{
+  CFPredict<NMFPolicy,
+            OverallMeanNormalization,
+            EuclideanSearch,
+            AverageInterpolation>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * SimilarityInterpolation.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictSimilarityInterpolation)
+{
+  CFPredict<NMFPolicy,
+            OverallMeanNormalization,
+            EuclideanSearch,
+            SimilarityInterpolation>(2.0);
+}
+
+/**
+ * Make sure that Predict() is returning reasonable results for
+ * RegressionInterpolation.
+ */
+BOOST_AUTO_TEST_CASE(CFPredictRegressionInterpolation)
+{
+  CFPredict<RegSVDPolicy,
+            OverallMeanNormalization,
+            EuclideanSearch,
+            RegressionInterpolation>(2.0);
+}
+
 BOOST_AUTO_TEST_SUITE_END();
