@@ -19,19 +19,19 @@
 #include <mlpack/methods/ann/loss_functions/mean_squared_error.hpp>
 #include <mlpack/methods/reinforcement_learning/q_learning.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/mountain_car.hpp>
-#include <mlpack/methods/reinforcement_learning/environment/acrobat.hpp>
+#include <mlpack/methods/reinforcement_learning/environment/acrobot.hpp>
 #include <mlpack/methods/reinforcement_learning/environment/cart_pole.hpp>
 #include <mlpack/methods/reinforcement_learning/policy/greedy_policy.hpp>
-#include <mlpack/core/optimizers/adam/adam_update.hpp>
-#include <mlpack/core/optimizers/rmsprop/rmsprop_update.hpp>
 #include <mlpack/methods/reinforcement_learning/training_config.hpp>
 
+#include <ensmallen.hpp>
+
 #include <boost/test/unit_test.hpp>
 #include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::ann;
-using namespace mlpack::optimization;
+using namespace ens;
 using namespace mlpack::rl;
 
 BOOST_AUTO_TEST_SUITE(QLearningTest);
@@ -173,70 +173,159 @@ BOOST_AUTO_TEST_CASE(CartPoleWithDoubleDQN)
   BOOST_REQUIRE(converged);
 }
 
-//! Test DQN in Acrobat task.
-BOOST_AUTO_TEST_CASE(AcrobatWithDQN)
+//! Test DQN in Acrobot task.
+BOOST_AUTO_TEST_CASE(AcrobotWithDQN)
 {
-    // Set up the network.
-  FFN<MeanSquaredError<>, GaussianInitialization> model(MeanSquaredError<>(),
-      GaussianInitialization(0, 0.001));
-  model.Add<Linear<>>(4, 64);
-  model.Add<ReLULayer<>>();
-  model.Add<Linear<>>(64, 32);
-  model.Add<ReLULayer<>>();
-  model.Add<Linear<>>(32, 3);
-
-  // Set up the policy and replay method.
-  GreedyPolicy<Acrobat> policy(1.0, 1000, 0.1);
-  RandomReplay<Acrobat> replayMethod(20, 10000);
-
-  TrainingConfig config;
-  config.StepSize() = 0.01;
-  config.Discount() = 0.99;
-  config.TargetNetworkSyncInterval() = 100;
-  config.ExplorationSteps() = 100;
-  config.DoubleQLearning() = false;
-  config.StepLimit() = 400;
-
-  // Set up DQN agent.
-  QLearning<Acrobat, decltype(model), AdamUpdate, decltype(policy)>
-      agent(std::move(config), std::move(model), std::move(policy),
-      std::move(replayMethod));
-
-  arma::running_stat<double> averageReturn;
-  size_t episodes = 0;
-  bool converged = true;
-  while (true)
+  // We will allow three trials, although it would be very uncommon for the test
+  // to use more than one.
+  bool success = false;
+  for (size_t trial = 0; trial < 3; ++trial)
   {
-    double episodeReturn = agent.Episode();
-    averageReturn(episodeReturn);
-    episodes += 1;
+    // Set up the network.
+    FFN<MeanSquaredError<>, GaussianInitialization> model(MeanSquaredError<>(),
+        GaussianInitialization(0, 0.001));
+    model.Add<Linear<>>(4, 64);
+    model.Add<ReLULayer<>>();
+    model.Add<Linear<>>(64, 32);
+    model.Add<ReLULayer<>>();
+    model.Add<Linear<>>(32, 3);
 
-    if (episodes > 1000)
+    // Set up the policy and replay method.
+    GreedyPolicy<Acrobot> policy(1.0, 1000, 0.1);
+    RandomReplay<Acrobot> replayMethod(20, 10000);
+
+    TrainingConfig config;
+    config.StepSize() = 0.01;
+    config.Discount() = 0.99;
+    config.TargetNetworkSyncInterval() = 100;
+    config.ExplorationSteps() = 100;
+    config.DoubleQLearning() = false;
+    config.StepLimit() = 400;
+
+    // Set up DQN agent.
+    QLearning<Acrobot, decltype(model), AdamUpdate, decltype(policy)>
+        agent(std::move(config), std::move(model), std::move(policy),
+        std::move(replayMethod));
+
+    arma::running_stat<double> averageReturn;
+    size_t episodes = 0;
+    bool converged = true;
+    while (true)
     {
-      Log::Debug << "Acrobat with DQN failed." << std::endl;
-      converged = false;
-      break;
+      double episodeReturn = agent.Episode();
+      averageReturn(episodeReturn);
+      episodes += 1;
+
+      if (episodes > 1000)
+      {
+        Log::Debug << "Acrobot with DQN failed." << std::endl;
+        converged = false;
+        break;
+      }
+
+      /**
+       * I am using a threshold of -380 to check convergence.
+       */
+      Log::Debug << "Average return: " << averageReturn.mean()
+          << " Episode return: " << episodeReturn << std::endl;
+      if (averageReturn.mean() > -380.00)
+      {
+        agent.Deterministic() = true;
+        arma::running_stat<double> testReturn;
+        for (size_t i = 0; i < 20; ++i)
+          testReturn(agent.Episode());
+
+        Log::Debug << "Average return in deterministic test: "
+            << testReturn.mean() << std::endl;
+        break;
+      }
     }
 
-    /**
-     * I am using a thresold of -380 to check convegence.
-     */
-    Log::Debug << "Average return: " << averageReturn.mean()
-        << " Episode return: " << episodeReturn << std::endl;
-    if (averageReturn.mean() > -380.00)
+    if (converged)
     {
-      agent.Deterministic() = true;
-      arma::running_stat<double> testReturn;
-      for (size_t i = 0; i < 20; ++i)
-        testReturn(agent.Episode());
-
-      Log::Debug << "Average return in deterministic test: "
-          << testReturn.mean() << std::endl;
+      success = true;
       break;
     }
   }
 
-  BOOST_REQUIRE(converged);
+  BOOST_REQUIRE_EQUAL(success, true);
+}
+
+//! Test DQN in Mountain Car task.
+BOOST_AUTO_TEST_CASE(MountainCarWithDQN)
+{
+  // We will allow three trials total.
+  bool success = false;
+  for (size_t trial = 0; trial < 3; trial++)
+  {
+    // Set up the network.
+    FFN<MeanSquaredError<>, GaussianInitialization> model(MeanSquaredError<>(),
+        GaussianInitialization(0, 0.001));
+    model.Add<Linear<>>(2, 64);
+    model.Add<ReLULayer<>>();
+    model.Add<Linear<>>(64, 32);
+    model.Add<ReLULayer<>>();
+    model.Add<Linear<>>(32, 3);
+
+    // Set up the policy and replay method.
+    GreedyPolicy<MountainCar> policy(1.0, 1000, 0.1);
+    RandomReplay<MountainCar> replayMethod(20, 10000);
+
+    TrainingConfig config;
+    config.StepSize() = 0.0001;
+    config.Discount() = 0.9;
+    config.TargetNetworkSyncInterval() = 100;
+    config.ExplorationSteps() = 100;
+    config.DoubleQLearning() = false;
+    config.StepLimit() = 400;
+
+    // Set up DQN agent.
+    QLearning<MountainCar, decltype(model), AdamUpdate, decltype(policy)>
+        agent(std::move(config), std::move(model), std::move(policy),
+        std::move(replayMethod));
+
+    arma::running_stat<double> averageReturn;
+    size_t episodes = 0;
+    bool converged = true;
+    while (true)
+    {
+      double episodeReturn = agent.Episode();
+      averageReturn(episodeReturn);
+      episodes += 1;
+
+      if (episodes > 1000)
+      {
+        Log::Debug << "Mountain Car with DQN failed." << std::endl;
+        converged = false;
+        break;
+      }
+
+      /**
+       * Set a threshold of -370 to check convergence.
+       */
+      Log::Debug << "Average return: " << averageReturn.mean()
+          << " Episode return: " << episodeReturn << std::endl;
+      if (averageReturn.mean() > -370)
+      {
+        agent.Deterministic() = true;
+        arma::running_stat<double> testReturn;
+        for (size_t i = 0; i < 10; ++i)
+          testReturn(agent.Episode());
+
+        Log::Debug << "Average return in deterministic test: "
+            << testReturn.mean() << std::endl;
+        break;
+      }
+    }
+
+    if (converged)
+    {
+      success = true;
+      break;
+    }
+  }
+
+  BOOST_REQUIRE_EQUAL(success, true);
 }
 
 BOOST_AUTO_TEST_SUITE_END();
